@@ -50,6 +50,8 @@ static const R_CallMethodDef callMethods[] = {
     {"R_ltMatrices_solve", (DL_FUNC) &R_ltMatrices_solve, 6},
     {"R_ltMatrices_tcrossprod", (DL_FUNC) &R_ltMatrices_tcrossprod , 6},
     {"R_ltMatrices_Mult", (DL_FUNC) &R_ltMatrices_Mult, 5},
+    {"R_ltMatrices_Mult_transpose", (DL_FUNC) &R_ltMatrices_Mult_transpose, 5},
+    {"R_ltMatrices_colSumsdnorm", (DL_FUNC) &R_ltMatrices_colSumsdnorm, 3},
     {"R_lpmvnorm", (DL_FUNC) &R_lpmvnorm, 11},
     {"R_slpmvnorm", (DL_FUNC) &R_slpmvnorm, 10},
     {"R_vectrick", (DL_FUNC) &R_vectrick, 7},
@@ -57,7 +59,6 @@ static const R_CallMethodDef callMethods[] = {
     {NULL, NULL, 0}
 };
 
-
 void attribute_visible R_init_mvtnorm(DllInfo *dll)
 {
     R_registerRoutines(dll, cMethods, callMethods, NULL, NULL);
@@ -68,6 +69,8 @@ void attribute_visible R_init_mvtnorm(DllInfo *dll)
     R_RegisterCCallable("mvtnorm", "R_ltMatrices_solve", (DL_FUNC) &R_ltMatrices_solve);
     R_RegisterCCallable("mvtnorm", "R_ltMatrices_tcrossprod", (DL_FUNC) &R_ltMatrices_tcrossprod);
     R_RegisterCCallable("mvtnorm", "R_ltMatrices_Mult", (DL_FUNC) &R_ltMatrices_Mult);
+    R_RegisterCCallable("mvtnorm", "R_ltMatrices_Mult_transpose", (DL_FUNC) &R_ltMatrices_Mult_transpose);
+    R_RegisterCCallable("mvtnorm", "R_ltMatrices_colSumsdnorm", (DL_FUNC) &R_ltMatrices_colSumsdnorm);
     R_RegisterCCallable("mvtnorm", "R_lpmvnorm", (DL_FUNC) &R_lpmvnorm);
     R_RegisterCCallable("mvtnorm", "R_slpmvnorm", (DL_FUNC) &R_slpmvnorm);
     R_RegisterCCallable("mvtnorm", "R_vectrick", (DL_FUNC) &R_vectrick);
